@@ -10,7 +10,7 @@
  * This function is called right before the kernel is booted. "blob" is the
  * device tree that will be passed to the kernel.
  */
-int ft_system_setup(void *blob, bd_t *bd)
+int ft_system_setup(void *blob, struct bd_info *bd)
 {
 	const char *gpu_compats[] = {
 #if defined(CONFIG_TEGRA124)
