@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0+
 /*
  * linux/arch/powerpc/kernel/traps.c
  *
@@ -12,8 +13,6 @@
  *
  * (C) Copyright 2000
  * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
- *
- * SPDX-License-Identifier:	GPL-2.0+
  */
 
 /*
@@ -21,7 +20,10 @@
  */
 
 #include <common.h>
+#include <asm/ptrace.h>
 #include <command.h>
+#include <init.h>
+#include <irq_func.h>
 #include <kgdb.h>
 #include <asm/processor.h>
 
@@ -286,11 +288,3 @@ void DebugException(struct pt_regs *regs)
 	do_bedbug_breakpoint( regs );
 #endif
 }
-
-/* Probe an address by reading.	 If not present, return -1, otherwise
- * return 0.
- */
-int addr_probe(uint *addr)
-{
-	return 0;
-}
