@@ -7,6 +7,9 @@
 #include <debug_uart.h>
 #include <dm.h>
 #include <hang.h>
+#include <image.h>
+#include <init.h>
+#include <log.h>
 #include <spl.h>
 #include <asm/cpu.h>
 #include <asm/mtrr.h>
