@@ -8,9 +8,11 @@
  */
 
 #include <common.h>
+#include <init.h>
 #include <pci.h>
 #include <asm/immap.h>
 #include <asm/io.h>
+#include <linux/delay.h>
 
 DECLARE_GLOBAL_DATA_PTR;
 
