@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0+
 /*
  * Imagination Technologies MIPSfpga platform code
  *
@@ -5,16 +6,19 @@
  *
  * Zubair Lutfullah Kakakhel <Zubair.Kakakhel@imgtec.com>
  *
- * SPDX-License-Identifier:	GPL-2.0+
- *
  */
 
 #include <common.h>
+#include <init.h>
+
+DECLARE_GLOBAL_DATA_PTR;
 
 /* initialize the DDR Controller and PHY */
-phys_size_t initdram(int board_type)
+int dram_init(void)
 {
 	/* MIG IP block is smart and doesn't need SW
 	 * to do any init */
-	return CONFIG_SYS_SDRAM_SIZE;	/* in bytes */
+	gd->ram_size = CONFIG_SYS_SDRAM_SIZE;	/* in bytes */
+
+	return 0;
 }
