@@ -6,6 +6,7 @@
 
 #include <common.h>
 #include <clk-uclass.h>
+#include <log.h>
 #include <dm/device.h>
 #include <dm/uclass.h>
 #include <dm/lists.h>
@@ -21,13 +22,13 @@ int clk_register(struct clk *clk, const char *drv_name,
 
 	ret = uclass_get_device_by_name(UCLASS_CLK, parent_name, &parent);
 	if (ret) {
-		printf("%s: name: %s parent: %s [0x%p]\n",
-		       __func__, name, parent->name, parent);
+		printf("%s: failed to get %s device (parent of %s)\n",
+		       __func__, parent_name, name);
+	} else {
+		debug("%s: name: %s parent: %s [0x%p]\n", __func__, name,
+		      parent->name, parent);
 	}
 
-	debug("%s: name: %s parent: %s [0x%p]\n", __func__, name, parent->name,
-	      parent);
-
 	drv = lists_driver_lookup_name(drv_name);
 	if (!drv) {
 		printf("%s: %s is not a valid driver name\n",
