@@ -7,8 +7,11 @@
 
 #include <common.h>
 #include <dm.h>
+#include <part.h>
 #include <video.h>
+#include <asm/cache.h>
 #include <dm/device_compat.h>
+#include <linux/delay.h>
 #include <linux/mbus.h>
 #include <asm/io.h>
 #include <asm/arch/cpu.h>
