@@ -32,7 +32,6 @@
 #define CONFIG_SH_ETHER_CACHE_WRITEBACK
 #define CONFIG_SH_ETHER_CACHE_INVALIDATE
 #define CONFIG_SH_ETHER_ALIGNE_SIZE	64
-#define CONFIG_BITBANGMII
 #define CONFIG_BITBANGMII_MULTI
 
 /* Board Clock */
@@ -45,7 +44,6 @@
 /* SPL support */
 #define CONFIG_SPL_STACK		0xe6340000
 #define CONFIG_SPL_MAX_SIZE		0x4000
-#define CONFIG_SYS_SPI_U_BOOT_OFFS	0x140000
 #ifdef CONFIG_SPL_BUILD
 #define CONFIG_CONS_SCIF2
 #define CONFIG_SH_SCIF_CLK_FREQ		65000000
