@@ -1,10 +1,9 @@
+/* SPDX-License-Identifier: BSD-2-Clause */
 /******************************************************************************
  * Copyright (c) 2004, 2008 IBM Corporation
  * Copyright (c) 2009 Pattrick Hueper <phueper@hueper.net>
  * All rights reserved.
  *
- * SPDX-License-Identifier:	BSD-2-Clause
- *
  * Contributors:
  *     IBM Corporation - initial implementation
  *****************************************************************************/
@@ -104,8 +103,6 @@ struct vbe_ddc_info {
 
 extern struct vbe_mode_info mode_info;
 
-struct graphic_device;
-int vbe_get_video_info(struct graphic_device *gdev);
 struct video_priv;
 struct video_uc_platdata;
 int vbe_setup_video_priv(struct vesa_mode_info *vesa,
